@@ -1,9 +1,11 @@
 # pylint: disable=missing-module-docstring
 import os
+import time
 import enum
 from datetime import datetime as dt
 from datetime import timedelta
 import logging
+import traceback
 
 from lxml import etree
 
@@ -21,7 +23,8 @@ class TAKClient:
 
     def __init__(self, router=None, cot_log_dir=None):
         self.router = router
-        self.user = models.TAKUser()
+        self.user = None
+        self.connected = time.time()
 
         self.cot_log_dir = cot_log_dir
         self.cot_fp = None
@@ -33,7 +36,10 @@ class TAKClient:
         self.lgr = logging.getLogger(TAKClient.__name__)
 
     def __repr__(self):
-        return f"<TAKClient uid={self.user.uid} " f"callsign={self.user.callsign}>"
+        if self.user:
+            return f"<TAKClient uid={self.user.uid} " f"callsign={self.user.callsign}>"
+
+        return "<TAKClient uid=None callsign=None>"
 
     def send(self, data):
         """
@@ -51,7 +57,7 @@ class TAKClient:
                 pass
             self.cot_fp = None
 
-    def log_event(self, evt):
+    def log_event(self, evt=None, elm=None, exc=None):
         """
         Writes the COT XML to the logfile, if configured.
 
@@ -61,10 +67,12 @@ class TAKClient:
         if not self.cot_log_dir:
             return
         # Skip logging of pings
-        if evt.uid.endswith("-ping"):
+        if evt and evt.uid and evt.uid.endswith("-ping"):
             return
         # Don't log if we don't have a user yet
-        if not self.user.uid:
+        if self.user is None or self.user.uid is None:
+            return
+        if evt is None and elm is None:
             return
 
         # Open the COT file if it's the first run
@@ -73,7 +81,7 @@ class TAKClient:
             #     : Could happen on WiFi -> LTE handoff
             name = os.path.join(self.cot_log_dir, f"{self.user.uid}.cot")
             try:
-                self.lgr.info("Opening logfile %s", name)
+                self.lgr.debug("Opening logfile %s", name)
                 self.cot_fp = open(name, "a+")
             except OSError as exc:
                 self.lgr.warning("Unable to open COT log: %s", exc)
@@ -82,8 +90,21 @@ class TAKClient:
                 return
 
         try:
-            elm = evt.as_element
-            self.cot_fp.write(etree.tostring(elm, pretty_print=True).decode())
+            if elm is None:
+                elm = evt.as_element
+
+            if exc:
+                taky_err = etree.Element("__taky_err")
+                taky_err.append(etree.Comment(exc))
+                elm.append(taky_err)
+
+            doc = etree.tostring(elm, pretty_print=True).decode()
+        except Exception as exc:
+            self.lgr.warning("Unable to build packet string for logfile", exc_info=exc)
+            return
+
+        try:
+            self.cot_fp.write(doc)
             self.cot_fp.flush()
         except (IOError, OSError) as exc:
             self.lgr.warning("Unable to write to COT log: %s", exc)
@@ -100,30 +121,30 @@ class TAKClient:
         for (_, elm) in self.xdc.read_events():
             try:
                 evt = models.Event.from_elm(elm)
+                if evt.etype == "t-x-c-t":
+                    self.pong()
+                    continue
+
+                if evt.etype.startswith("a"):
+                    self.handle_atom(evt)
+
+                self.router.route(self, evt)
                 self.log_event(evt)
             except models.UnmarshalError as exc:
-                self.lgr.warning("Unable to parse Event: %s", exc)
+                self.lgr.debug("Unable to parse Event: %s", exc, exc_info=exc)
                 self.lgr.debug(etree.tostring(elm, pretty_print=True))
-                # TODO: Call log_event(elm, failed=True, comment=str(exc))
+                self.log_event(elm=elm, exc=traceback.format_exc())
                 continue
             except Exception as exc:  # pylint: disable=broad-except
-                self.lgr.error("Unhandled exception parsing Event: %s", exc)
-                self.lgr.debug(etree.tostring(elm, pretty_print=True))
-                # TODO: Call log_event(elm, failed=True, comment=str(exc))
+                self.lgr.error(
+                    "Unhandled exception parsing Event: %s", exc, exc_info=exc
+                )
+                self.lgr.error(etree.tostring(elm, pretty_print=True))
+                self.log_event(elm=elm, exc=traceback.format_exc())
                 continue
             finally:
                 elm.clear(keep_tail=True)
 
-            # Hard stop processing on a tak ping
-            if evt.etype == "t-x-c-t":
-                self.pong()
-                return
-
-            if evt.etype.startswith("a"):
-                self.handle_atom(evt)
-
-            self.router.route(self, evt)
-
     def handle_atom(self, evt):
         """
         Process a COT atom.
@@ -134,10 +155,12 @@ class TAKClient:
         if evt.detail is None:
             return
 
-        if evt.detail.elm.find("takv") is not None:
-            first_ident = self.user.update_from_evt(evt)
-            if first_ident:
+        if isinstance(evt.detail, models.TAKUser):
+            if self.user is None:
+                self.user = evt.detail
                 self.router.client_ident(self)
+            else:
+                self.user = evt.detail
 
     def pong(self):
         """
@@ -177,9 +200,16 @@ class SocketTAKClient(TAKClient):
         self.out_buff = b""
 
     def __repr__(self):
+        if self.user:
+            return (
+                f"<SocketTAKClient uid={self.user.uid} "
+                f"callsign={self.user.callsign} "
+                f"addr={self.addr[0]}:{self.addr[1]}>"
+            )
+
         return (
-            f"<SocketTAKClient uid={self.user.uid} "
-            f"callsign={self.user.callsign} "
+            f"<SocketTAKClient uid=None "
+            f"callsign=None "
             f"addr={self.addr[0]}:{self.addr[1]}>"
         )
 
