@@ -1,4 +1,5 @@
 # pylint: disable=missing-module-docstring
+import time
 import enum
 import logging
 
@@ -28,8 +29,15 @@ class COTRouter:
         #     : should prohibit multiple sockets sharing a client
         self.clients = set()
         self.persist = build_persistence(config)
+        self.last_prune = 0
         self.lgr = logging.getLogger(self.__class__.__name__)
 
+    def prune(self):
+        now = time.time()
+        if (now - self.last_prune) > 10:
+            self.last_prune = now
+            self.persist.prune()
+
     def client_connect(self, client):
         """
         Add a client to the router
@@ -48,7 +56,7 @@ class COTRouter:
         """
         self.lgr.debug("Sending persistence objects to %s", client)
         for event in self.persist.get_all():
-            if event.uid == client.user.uid:
+            if client.user and event.uid == client.user.uid:
                 continue
 
             client.send(event)
@@ -58,6 +66,9 @@ class COTRouter:
         Search the client database for a requested client
         """
         for client in self.clients:
+            if not client.user:
+                continue
+
             if uid and client.user.uid == uid:
                 return client
             if callsign and client.user.callsign == callsign:
@@ -69,7 +80,11 @@ class COTRouter:
         """
         Broadcast a message from source to all clients
         """
-        self.lgr.debug("%s -> Broadcast: %s", src.user.callsign, msg)
+        if src.user:
+            self.lgr.debug("%s -> Broadcast: %s", src.user.callsign, msg)
+        else:
+            self.lgr.debug("Anonymous Broadcast: %s", msg)
+
         self.persist.track(msg)
         for client in self.clients:
             if client is src:
@@ -83,18 +98,22 @@ class COTRouter:
 
         If group is not specified, the source's group is used.
         """
+        if isinstance(src, TAKClient):
+            src = src.user
+
         if group is None:
-            if isinstance(src, models.TAKUser):
-                group = src.group
-            elif isinstance(src, TAKClient):
-                group = src.user.group
-            else:
+            if src is None:
                 raise ValueError("Unable to determine group to send to")
+            group = src.group
 
         if not isinstance(group, models.Teams):
             raise ValueError("group must be models.Teams")
 
-        self.lgr.debug("%s -> %s: %s", src.user.callsign, group, msg)
+        if src:
+            self.lgr.debug("%s -> %s: %s", src.callsign, group, msg)
+        else:
+            self.lgr.debug("Anonymous -> %s: %s", group, msg)
+
         for client in self.clients:
             if client.user is src:
                 continue
@@ -121,7 +140,7 @@ class COTRouter:
                 if client:
                     self.lgr.debug(
                         "%s -> %s: (geochat) %s",
-                        src.user.callsign,
+                        chat.src_cs,
                         client.user.callsign,
                         chat.message,
                     )
@@ -136,12 +155,19 @@ class COTRouter:
             for callsign in evt.detail.marti_cs:
                 client = self.find_client(callsign=callsign)
                 if client:
-                    self.lgr.debug(
-                        "%s -> %s (marti): %s",
-                        src.user.callsign,
-                        client.user.callsign,
-                        evt,
-                    )
+                    if src.user:
+                        self.lgr.debug(
+                            "%s -> %s (marti): %s",
+                            src.user.callsign,
+                            client.user.callsign,
+                            evt,
+                        )
+                    else:
+                        self.lgr.debug(
+                            "Anonymous -> %s (marti): %s",
+                            client.user.callsign,
+                            evt,
+                        )
                     client.send(evt)
             return
 
